@@ -68,6 +68,26 @@ typedef gss_union_ctx_id_desc *gss_union_ctx_id_t;
 #define SVC_RPC_GSS_FLAG_NONE    0x0000
 #define SVC_RPC_GSS_FLAG_MSPAC   0x0001
 
+/* This defines the sequence window for SVC-GSS implementation.
+ * Due to the way we store the sequence window mask, this number MUST always
+ * be a multiple of CHAR_BIT.
+ */
+#define SVC_GSS_SEQ_WIN 256
+
+/* We additionally maintain an internal sequence window, that is double the
+ * size of the sequence window that is published to the client. This is done
+ * because, due to the asynchronous (= unordered) nature of request processing
+ * at the server, it is possible that in a window of N, assuming sequence
+ * numbers from 1 to N, the server processes and responds to the request with
+ * (N+1) sequence number before the request with sequence number `1` (that is,
+ * the server responds to `N`th request, then the client sends `N+1`th request,
+ * which also gets processed before the older request with sequence number `1`)
+ * In such a scenario, the request with sequence number `1` will fall outside
+ * the sequence window, and will be dropped. In order to reduce the chances of
+ * such a situation, we maintain a larger sequence window on the server
+ */
+#define SVC_GSS_SEQ_WIN_INTERNAL (SVC_GSS_SEQ_WIN * 2)
+
 struct svc_rpc_gss_data {
 	struct opr_rbtree_node node_k;
 	TAILQ_ENTRY(svc_rpc_gss_data) lru_q;
@@ -82,10 +102,8 @@ struct svc_rpc_gss_data {
 	gss_ctx_id_t ctx;	/* context id */
 	struct rpc_gss_sec sec;	/* security triple */
 	gss_buffer_desc cname;	/* GSS client name */
-	u_int seq;
-	u_int win;
 	u_int seqlast;
-	uint32_t seqmask;
+	uint8_t seqmask[SVC_GSS_SEQ_WIN_INTERNAL / CHAR_BIT];
 	gss_name_t client_name;
 	gss_buffer_desc checksum;
 	struct {
@@ -129,10 +147,12 @@ unref_svc_rpc_gss_data(struct svc_rpc_gss_data *gd)
 struct svc_rpc_gss_data *authgss_ctx_hash_get(struct rpc_gss_cred *gc);
 bool authgss_ctx_hash_set(struct svc_rpc_gss_data *gd);
 bool authgss_ctx_hash_del(struct svc_rpc_gss_data *gd);
+void authgss_ctx_hash_clear(void);
 
 bool svcauth_gss_acquire_cred(void);
 bool svcauth_gss_release_cred(void);
 bool svcauth_gss_import_name(char *service);
 bool svcauth_gss_set_svc_name(gss_name_t name);
+void svcauth_gss_set_status(bool status_enabled);
 
 #endif				/* GSS_INTERNAL_H */
