@@ -77,6 +77,8 @@
 #endif
 #include "svc_ioq.h"
 
+#include "metrics_libntirpc.h"
+
 #define SVC_VERSQUIET 0x0001	/* keep quiet about vers mismatch */
 #define version_keepquiet(xp) ((u_long)(xp)->xp_p3 & SVC_VERSQUIET)
 
@@ -130,6 +132,9 @@ svc_init(svc_init_params *params)
 	struct work_pool_params work_pool_params = {0,};
 	uint32_t channels = params->channels ? params->channels : 8;
 
+	/* Initialize metrics for libntirpc */
+	metrics_libntirpc_init();
+
 	mutex_lock(&__svc_params->mtx);
 	if (__svc_params->initialized) {
 		__warnx(TIRPC_DEBUG_FLAG_WARN,
