@@ -213,6 +213,8 @@ svc_dg_ncreatef(const int fd, const u_int sendsz, const u_int recvsz,
 				    RPC_DPLX_LOCKED |
 				    SVC_RQST_FLAG_CHAN_AFFINITY);
 
+	atomic_set_uint16_t_bits(&xprt->xp_flags, SVC_XPRT_FLAG_READY);
+
 	/* release */
 	rpc_dplx_rui(rec);
 	XPRT_TRACE(xprt, __func__, __func__, __LINE__);
@@ -253,7 +255,7 @@ svc_dg_rendezvous(SVCXPRT *xprt)
 	su->su_dr.maxrec = req_su->su_dr.maxrec;
 	svc_dg_override_ops(newxprt, xprt);
 
- again:
+again:
 	iov.iov_base = &su[1];
 	iov.iov_len = su->su_dr.maxrec;
 	mesgp = &su->su_msghdr;
@@ -279,21 +281,21 @@ svc_dg_rendezvous(SVCXPRT *xprt)
 		return (XPRT_DIED);
 	}
 
-        if (sp->sa_family == (sa_family_t) 0xffff) {
-                __warnx(TIRPC_DEBUG_FLAG_ERROR,
-                        "%s: Bad message sa_family is 0xffff",
-                        __func__);
+	if (sp->sa_family == (sa_family_t) 0xffff) {
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: xprt(%p) newxprt(%p) fd %d Bad message sa_family is 0xffff",
+			__func__, xprt, newxprt, newxprt->xp_fd);
 		svc_dg_xprt_free(su);
-                return SVC_STAT(xprt);
-        }
+		return SVC_STAT(xprt);
+	}
 
-        if (rlen == -1 || (rlen < (ssize_t) (4 * sizeof(u_int32_t)))) {
-                __warnx(TIRPC_DEBUG_FLAG_ERROR,
-                        "%s: Bad message rlen: %d",
-                        __func__, rlen);
+	if (rlen == -1 || (rlen < (ssize_t) (4 * sizeof(u_int32_t)))) {
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+				"%s: xprt(%p) newxprt(%p) fd %d Bad message rlen: %d",
+				__func__, xprt, newxprt, newxprt->xp_fd, rlen);
 		svc_dg_xprt_free(su);
-                return SVC_STAT(xprt);
-        }
+		return SVC_STAT(xprt);
+	}
 
 	__rpc_address_setup(&newxprt->xp_local);
 	__rpc_address_setup(&newxprt->xp_remote);
@@ -313,10 +315,14 @@ svc_dg_rendezvous(SVCXPRT *xprt)
 #endif
 
 	xdrmem_create(su->su_dr.ioq.xdrs, iov.iov_base, iov.iov_len,
-		      XDR_DECODE);
+		XDR_DECODE);
 
 	SVC_REF(xprt, SVC_REF_FLAG_NONE);
 	newxprt->xp_parent = xprt;
+
+	atomic_set_uint16_t_bits(&newxprt->xp_flags,
+		SVC_XPRT_FLAG_READY);
+
 	return (xprt->xp_dispatch.rendezvous_cb(newxprt));
 }
 
@@ -401,7 +407,7 @@ svc_dg_reply(struct svc_req *req)
 	struct cmsghdr* cmsg;
 	struct iovec iov;
 	size_t slen;
-        char msg_control[sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo)];
+        char msg_control[sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo)] = {0};
 
 	if (!xprt->xp_remote.nb.len) {
 		__warnx(TIRPC_DEBUG_FLAG_WARN,
